package ru.yandex.webmaster3.storage.download.searchquery;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.joda.time.LocalDate;
import ru.yandex.webmaster3.core.data.WebmasterHostId;
import ru.yandex.webmaster3.core.download.FileFormat;
import ru.yandex.webmaster3.core.searchquery.OrderDirection;
import ru.yandex.webmaster3.core.searchquery.QueryIndicator;
import ru.yandex.webmaster3.core.searchquery.SpecialGroup;
import ru.yandex.webmaster3.core.worker.task.WorkerTaskData;
import ru.yandex.webmaster3.core.worker.task.WorkerTaskType;
import ru.yandex.webmaster3.storage.searchquery.DeviceType;
import ru.yandex.webmaster3.storage.searchquery.RegionInclusion;

import java.util.UUID;

/**
 * @author lester
 */
public class DownloadUrlsStatisticTaskData extends DownloadGroupStatisticTaskData {
    private final SpecialGroup specialGroup;

    public DownloadUrlsStatisticTaskData(@JsonProperty("taskId") UUID taskId,
                                         @JsonProperty("hostId") WebmasterHostId hostId,
                                         @JsonProperty("localDateFrom") LocalDate localDateFrom,
                                         @JsonProperty("localDateTo") LocalDate localDateTo,
                                         @JsonProperty("regionId") Integer[] regionId,
                                         @JsonProperty("regionInclusion") RegionInclusion regionInclusion,
                                         @JsonProperty("specialGroup") SpecialGroup specialGroup,
                                         @JsonProperty("deviceType") DeviceType deviceType,
                                         @JsonProperty("orderByInd") QueryIndicator orderByInd,
                                         @JsonProperty("orderDirectionInd") OrderDirection orderDirectionInd,
                                         @JsonProperty("exportFormat") FileFormat exportFormat,
                                         @JsonProperty("fileName") String fileName,
                                         @JsonProperty("hash") Long hash) {
        super(taskId, hostId, localDateFrom, localDateTo, regionId, regionInclusion, deviceType, orderByInd, orderDirectionInd, exportFormat, fileName, hash);
        this.specialGroup = specialGroup;
    }

    public DownloadUrlsStatisticTaskData(WebmasterHostId hostId,
                                         LocalDate localDateFrom,
                                         LocalDate localDateTo,
                                         Integer[] regionId,
                                         RegionInclusion regionInclusion,
                                         SpecialGroup specialGroup,
                                         DeviceType deviceType,
                                         QueryIndicator orderByInd,
                                         OrderDirection orderDirectionInd,
                                         FileFormat exportFormat,
                                         String fileName,
                                         Long hash) {
        super(hostId, localDateFrom, localDateTo, regionId, regionInclusion, deviceType, orderByInd, orderDirectionInd, exportFormat, fileName, hash);
        this.specialGroup = specialGroup;
    }

    public static WorkerTaskData createTaskData(WebmasterHostId hostId,
                                                LocalDate localDateFrom,
                                                LocalDate localDateTo,
                                                Integer[] regionId,
                                                RegionInclusion regionInclusion,
                                                SpecialGroup specialGroup,
                                                DeviceType deviceType,
                                                QueryIndicator orderByInd,
                                                OrderDirection orderDirectionInd,
                                                FileFormat exportFormat,
                                                String fileName,
                                                Long hash) {
        return new DownloadUrlsStatisticTaskData(hostId,
                localDateFrom,
                localDateTo,
                regionId,
                regionInclusion,
                specialGroup,
                deviceType,
                orderByInd,
                orderDirectionInd,
                exportFormat,
                fileName,
                hash);
    }

    public SpecialGroup getSpecialGroup() {
        return specialGroup;
    }

    @Override
    public WorkerTaskType getTaskType() {
        return WorkerTaskType.DOWNLOAD_STATISTIC_ALL_URLS_ALL_INDICATORS;
    }

    @Override
    public String getShortDescription() {
        return "Download all urls all indicators";
    }
}
