package ru.yandex.wmconsole.servantlet.top.queries;

import java.util.Arrays;
import java.util.List;

import org.springframework.beans.factory.annotation.Required;

import ru.yandex.common.framework.core.ServRequest;
import ru.yandex.common.framework.core.ServResponse;
import ru.yandex.wmconsole.data.info.HostDbHostInfo;
import ru.yandex.wmconsole.servantlet.WMCAuthorizedHostOperationServantlet;
import ru.yandex.wmconsole.service.HostDbHostInfoService;
import ru.yandex.wmconsole.service.WMCTopInfoService;
import ru.yandex.wmtools.common.error.InternalException;
import ru.yandex.wmtools.common.error.UserException;
import ru.yandex.wmtools.common.error.UserProblem;

public class RemoveCustomQueryServantlet extends WMCAuthorizedHostOperationServantlet {
    private static final String PARAM_QUERY_ID = "query_id";

    private WMCTopInfoService topInfoService;
    private HostDbHostInfoService hostDbHostInfoService;

    @Override
    public void doProcess(ServRequest req, ServResponse res, long userId) throws InternalException, UserException {

        List<Long> selectedQueryIds = Arrays.asList(getMultiParamLong(req, PARAM_QUERY_ID));
        if (selectedQueryIds.size() <= 0) {
            throw new UserException(UserProblem.REQUIRED_PARAM_MISSED, "Required parameter " + PARAM_QUERY_ID + " missed", PARAM_QUERY_ID);
        }

        HostDbHostInfo hostDbHostInfo = hostDbHostInfoService.getHostDbHostInfo(
                getHostInfoAndVerify(req, userId).getName());

        topInfoService.removeCustomQueriesForHost(hostDbHostInfo, selectedQueryIds);
    }

    @Required
    public void setTopInfoService(WMCTopInfoService topInfoService) {
        this.topInfoService = topInfoService;
    }

    @Required
    public void setHostDbHostInfoService(HostDbHostInfoService hostDbHostInfoService) {
        this.hostDbHostInfoService = hostDbHostInfoService;
    }
}
